// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Appearance class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class Appearance extends VrmlObject
{
	private EventInSFNode	_set_material;
	private EventInSFNode	_set_texture;
	private EventInSFNode	_set_textureTransform;

	private Material	_material = null;
	private Texture		_texture = null;
	private TextureTransform _textureTransform = null;

	// constructor builds a copy of itself and is added to the scene
	// graph.
	public Appearance()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Appearance {}")[0];

		createFields();
	}

	public Appearance(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Appearance {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_material = (EventInSFNode)node.getEventIn("material");
		_set_texture = (EventInSFNode)node.getEventIn("texture");
		_set_textureTransform = (EventInSFNode)node.getEventIn("textureTransform");

		have_browser = true;
	}

	// memory cleanup and object deallocation
	public void cleanup()
	{
		if(have_browser)
		{
			_set_material.setValue(null);
			_set_texture.setValue(null);
			_set_textureTransform.setValue(null);
		}

		if(_material != null)
			_material.cleanup();

		if(_texture != null)
			_texture.cleanup();

		if(_textureTransform != null)
			_textureTransform.cleanup();

		_material = null;
		_texture = null;
		_textureTransform = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "appearance ");
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Appearance {");

		// print the contents of the shape node
		if(_material != null)
			_material.writeToFile(fp, indent + 1);
		if(_texture != null)
			_texture.writeToFile(fp, indent + 1);
		if(_textureTransform != null)
			_textureTransform.writeToFile(fp, indent + 1);

		fp.println(buffer.toString() + "}");
	}

	public void set_material(Material mat)
	{
		if(_material != null)
			_material.cleanup();

		_material = mat;
		if(have_browser)
			_set_material.setValue(mat.node);
	}

	public Material get_material()
	{
		return(_material);
	}

	public void set_texture(Texture tex)
	{
		if(_texture != null)
			_texture.cleanup();

		_texture = tex;
		if(have_browser)
			_set_texture.setValue(tex.node);
	}

	public Texture get_texture()
	{
		return(_texture);
	}

	public void set_textureTransform(TextureTransform tex)
	{
		if(_textureTransform != null)
			_textureTransform.cleanup();

		_textureTransform = tex;
		if(have_browser)
			_set_textureTransform.setValue(tex.node);
	}

	public TextureTransform get_textureTransform()
	{
		return(_textureTransform);
	}

	public boolean hasMaterial()
	{
		return(_material != null);
	}

	public boolean hasTexture()
	{
		return(_texture != null);
	}

	public boolean hasTextureTransform()
	{
		return(_textureTransform != null);
	}
}
