// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Box class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class Box extends Geometry
{
	private float[] _size = null;

	// constructor builds a default box node
	public Box()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Box {}")[0];
		type = vlc.vrml.VrmlTypes.Box;
	}

	public Box(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Box {}")[0];
		type = vlc.vrml.VrmlTypes.Box;
	}

	public Box(float x, float y, float z)
	{
		Browser b = Browser.getBrowser();

		_size = new float[3];

		_size[0] = x;
		_size[1] = y;
		_size[2] = z;

		if(b == null)
			return;

		node = b.createVrmlFromString("Box { size " +
									   x + " " +
									   y + " " +
									   z + "}")[0];
		have_browser = true;
		type = vlc.vrml.VrmlTypes.Box;
	}

	public Box(Browser b, float x, float y, float z)
	{
		_size = new float[3];

		_size[0] = x;
		_size[1] = y;
		_size[2] = z;

		if(b == null)
			return;

		node = b.createVrmlFromString("Box { size " +
									   x + " " +
									   y + " " +
									   z + "}")[0];
		have_browser = true;
		type = vlc.vrml.VrmlTypes.Box;
	}

	public void cleanup()
	{
		_size = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "geometry ");
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.print("Box {");

		if(_size != null)
			fp.print(" size " +
					 _size[0] + " " +
					 _size[1] + " " +
					 _size[2]);

		fp.println("}");
	}

	public float[] get_size()
	{
		return(_size);
	}
}
