// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// CylinderSensor class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class CylinderSensor extends GeometrySensor
{
	private EventInSFFloat	_set_diskAngle;
	private EventInSFFloat	_set_maxAngle;
	private EventInSFFloat	_set_minAngle;

	private float	_diskAngle = 0.262f;
	private float	_maxAngle = -1;
	private float	_minAngle = 0;
	
	public CylinderSensor()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("CylinderSensor {}")[0];

		createFields();
	}

	public CylinderSensor(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("CylinderSensor {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_enabled = (EventInSFBool)node.getEventIn("enabled");
		_set_diskAngle = (EventInSFFloat)node.getEventIn("diskAngle");
		_set_maxAngle = (EventInSFFloat)node.getEventIn("maxAngle");
		_set_minAngle = (EventInSFFloat)node.getEventIn("minAngle");
		_set_offset = (EventInSFRotation)node.getEventIn("offset");
		_set_autoOffset = (EventInSFBool)node.getEventIn("autoOffset");

		_get_isActive = (EventOutSFBool)node.getEventOut("isActive");
		_get_rotation_changed = (EventOutSFRotation)node.getEventOut("rotation_changed");
		_get_trackPoint_changed = (EventOutSFVec3f)node.getEventOut("trackPoint_changed");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.CylinderSensor;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "CylinderSensor {");

		if(!_enabled)
			fp.println(buffer.toString() + "  enabled FALSE");
			
		if(_diskAngle != 0.262f)
			fp.println(buffer.toString() + "  diskAngle " + _diskAngle);
		
		if(_maxAngle != -1)
			fp.println(buffer.toString() + "  maxAngle " + _maxAngle);
			
		if((_offset[0] != 0) || (_offset[1] != 1) ||
		   (_offset[2] != 0) || (_offset[3] != 0))
		   	fp.println(buffer.toString() + "  offset " +
		   	           _offset[0] + " " +
		   	           _offset[1] + " " +
		   	           _offset[2] + " " +
		   	           _offset[3]);
		   	           
		if(!_autoOffset)
			fp.println(buffer.toString() + "  autoOffset FALSE");

		fp.println(buffer.toString() + "}");
	}
	
	public void set_diskAngle(float angle)
	{
		_diskAngle = angle;
		if(have_browser)
			_set_diskAngle.setValue(angle);
	}

	public float get_diskAngle()
	{
		return(_diskAngle);
	}
	
	public void set_maxAngle(float angle)
	{
		_maxAngle = angle;
		if(have_browser)
			_set_maxAngle.setValue(angle);
	}

	public float get_maxAngle()
	{
		return(_maxAngle);
	}
	
	public void set_minAngle(float angle)
	{
		_minAngle = angle;
		if(have_browser)
			_set_minAngle.setValue(angle);
	}

	public float get_minAngle()
	{
		return(_minAngle);
	}
}