// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// ElevationGrid class.
// If you don't want to specify any of the MFint fields then just set the
// parameter to be null, and the system will handle it.

package vlc.vrml.geometry;

import java.io.PrintStream;
import vrml.external.*;
import vrml.external.field.*;

public class ElevationGrid extends VrmlObject
{
	private EventInMFFloat	_set_height;
	private EventInSFNode	_set_color;
	private EventInSFNode	_set_texCoord;
	private EventInSFNode	_set_normal;

	private TextureCoordinate	_texCoord = null;
	private Color	_color = null;
	private Normal	_normal = null;
	private float[]	_height = null;
	private boolean	_ccw = true;
	private boolean	_colorPerVertex = true;
	private float	_creaseAngle = 0;
	private boolean	_normalPerVertex = true;
	private boolean	_solid = true;
	private int		_xDimension = 0;
	private float	_xSpacing = 0;
	private int		_zDimension = 0;
	private float	_zSpacing = 0;

	public ElevationGrid()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("ElevationGrid {}")[0];

		createFields();
	}

	public ElevationGrid(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("ElevationGrid {}")[0];

		createFields();
	}

	public ElevationGrid(float[] height,
						 boolean ccw,
						 boolean colorPerVertex,
						 float   creaseAngle,
						 boolean normalPerVertex,
						 boolean solid,
						 int     xDimension,
						 float   xSpacing,
						 int     zDimension,
						 int     zSpacing)
	{
		Browser b = Browser.getBrowser();

		_ccw = ccw;
		_colorPerVertex = colorPerVertex;
		_creaseAngle = creaseAngle;
		_normalPerVertex = normalPerVertex;
		_solid = solid;
		_xDimension = xDimension;
		_xSpacing = xSpacing;
		_zDimension = zDimension;
		_zSpacing = zSpacing;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	public ElevationGrid(Browser b,
						 float[] height,
						 boolean ccw,
						 boolean colorPerVertex,
						 float   creaseAngle,
						 boolean normalPerVertex,
						 boolean solid,
						 int     xDimension,
						 float   xSpacing,
						 int     zDimension,
						 int     zSpacing)
	{
		_ccw = ccw;
		_colorPerVertex = colorPerVertex;
		_creaseAngle = creaseAngle;
		_normalPerVertex = normalPerVertex;
		_solid = solid;
		_xDimension = xDimension;
		_xSpacing = xSpacing;
		_zDimension = zDimension;
		_zSpacing = zSpacing;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	private void createFields()
	{
		_set_color = (EventInSFNode)node.getEventIn("color");
		_set_texCoord = (EventInSFNode)node.getEventIn("texCoord");
		_set_normal = (EventInSFNode)node.getEventIn("normal");
		_set_height = (EventInMFFloat)node.getEventIn("set_height");

		have_browser = true;
	}

	private String createString()
	{
		int	i;
		String vrml_string = "ElevationGrid {";

		if(!_ccw)
			vrml_string += " ccw FALSE";

		if(!_colorPerVertex)
			vrml_string += " colorPerVertex FALSE";

		if(_creaseAngle != 0);
			vrml_string += " creaseAngle " + _creaseAngle;

		if(!_normalPerVertex)
			vrml_string += " normalPerVertex FALSE";

		if(!_solid)
			vrml_string += " solid FALSE";

		if(_height != null)
		{
			vrml_string += " height [";
			for(i = 0; i < _height.length; i++)
				vrml_string += " " + _height[i];

			vrml_string += " ]";
		}

		if(_xDimension != 0)
			vrml_string += " xDimension " + _xDimension;

		if(_xSpacing != 0)
			vrml_string += " xSpacing " + _xSpacing;

		if(_zDimension != 0)
			vrml_string += " zDimension " + _zDimension;

		if(_zSpacing != 0)
			vrml_string += " zSpacing " + _zSpacing;

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		if(_normal != null)
			_normal.cleanup();

		if(_color != null)
			_color.cleanup();

		if(_texCoord != null)
			_texCoord.cleanup();

		if(have_browser)
		{
			_set_texCoord.setValue(null);
			_set_color.setValue(null);
			_set_normal.setValue(null);
		}
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("ElevationGrid {");

		if(!_ccw)
			fp.println(buffer.toString() + "  ccw FALSE");

		if(_height != null)
		{
			fp.print(buffer.toString() + "  height ");

			if(_height.length == 1)
				fp.println(_height[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _height.length; i++)
					fp.println(buffer.toString() + "    " + _height[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(!_colorPerVertex)
			fp.println(buffer.toString() + "  colorPerVertex FALSE");


		if(_creaseAngle != 0);
			fp.println(buffer.toString() + "  creaseAngle " + _creaseAngle);


		if(!_normalPerVertex)
			fp.println(buffer.toString() + "  normalPerVertex FALSE");

		if(!_solid)
			fp.println(buffer.toString() + "  solid FALSE");

		if(_xDimension != 0)
			fp.println(buffer.toString() + " xDimension " + _xDimension);

		if(_xSpacing != 0)
			fp.println(buffer.toString() + " xSpacing " + _xSpacing);

		if(_zDimension != 0)
			fp.println(buffer.toString() + " zDimension " + _zDimension);

		if(_zSpacing != 0)
			fp.println(buffer.toString() + " zSpacing " + _zSpacing);

		if(_color != null)
			_color.writeToFile(fp, indent + 1);

		if(_texCoord != null)
			_texCoord.writeToFile(fp, indent + 1);

		if(_normal != null)
			_normal.writeToFile(fp, indent + 1);

		fp.println(buffer.toString() + "}");
	}

	public void set_color(Color color)
	{
		if(_color != null)
			_color.cleanup();

		_color = color;
		if(have_browser)
			_set_color.setValue(color.node);
	}

	public Color get_color()
	{
		return(_color);
	}

	public void set_texCoord(TextureCoordinate tex)
	{
		if(_texCoord != null)
			_texCoord.cleanup();

		_texCoord = tex;
		if(have_browser)
			_set_texCoord.setValue(tex.node);
	}

	public TextureCoordinate get_texCoord()
	{
		return(_texCoord);
	}

	public void set_normal(Normal normal)
	{
		if(_normal != null)
			_normal.cleanup();

		_normal = normal;
		if(have_browser)
			_set_normal.setValue(normal.node);
	}

	public Normal get_normal()
	{
		return(_normal);
	}

	public boolean get_ccw()
	{
		return(_ccw);
	}

	public boolean get_colorPerVertex()
	{
		return(_colorPerVertex);
	}

	public float get_creaseAngle()
	{
		return(_creaseAngle);
	}

	public boolean get_normalPerVertex()
	{
		return(_normalPerVertex);
	}

	public boolean get_solid()
	{
		return(_solid);
	}

	public void set_height(float[] vals)
	{
		_height = vals;
		if(have_browser)
			_set_height.setValue(vals);
	}

	public float[] get_height()
	{
		return(_height);
	}

	public int get_xDimension()
	{
		return(_xDimension);
	}

	public float get_xSpacing()
	{
		return(_xSpacing);
	}

	public int get_zDimension()
	{
		return(_zDimension);
	}

	public float get_zSpacing()
	{
		return(_zSpacing);
	}
}
