// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Extrusion class.
// If you don't want to specify any of the MFint fields then just set the
// parameter to be null, and the system will handle it.

package vlc.vrml.external.geometry;

import java.io.PrintStream;
import vrml.external.*;
import vrml.external.field.*;

public class Extrusion extends VrmlObject
{
	private EventInMFVec2f	_set_crossSection;
	private EventInMFRotation	_set_orientation;
	private EventInMFVec2f	_set_scale;
	private EventInMFVec3f	_set_spine;

	private boolean		_beginCap = true;
	private boolean		_ccw = true;
	private boolean		_convex = true;
	private float		_creaseAngle = 0;
	private float[][]	_crossSection;
	private boolean		_endCap = true;
	private float[][]	_orientation;
	private float[][]	_scale;
	private boolean 	_solid = true;
	private float[][] 	_spine = {{0, 0, 0}, { 0, 1, 0}};

	public Extrusion()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Extrusion {}")[0];

		createFields();
	}

	public Extrusion(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Extrusion {}")[0];

		createFields();
	}

	public Extrusion(boolean   beginCap,
					 boolean   ccw,
					 boolean   convex,
					 float     creaseAngle,
					 float[][] crossSection,
					 boolean   endCap,
					 float[][] orientation,
					 float[][] scale,
					 boolean   solid,
					 float[][] spine)
	{
		_beginCap = beginCap;
		_ccw = ccw;
		_convex = convex;
		_creaseAngle = creaseAngle;
		_crossSection = crossSection;
		_endCap = endCap;
		_orientation = orientation;
		_scale = scale;
		_solid = solid;
		_spine = spine;

		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	public Extrusion(Browser   b,
					 boolean   beginCap,
					 boolean   ccw,
					 boolean   convex,
					 float     creaseAngle,
					 float[][] crossSection,
					 boolean   endCap,
					 float[][] orientation,
					 float[][] scale,
					 boolean   solid,
					 float[][] spine)
	{
		_beginCap = beginCap;
		_ccw = ccw;
		_convex = convex;
		_creaseAngle = creaseAngle;
		_crossSection = crossSection;
		_endCap = endCap;
		_orientation = orientation;
		_scale = scale;
		_solid = solid;
		_spine = spine;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	private void createFields()
	{
		_set_crossSection = (EventInMFVec2f)node.getEventIn("set_crossSection");
		_set_orientation = (EventInMFRotation)node.getEventIn("set_orientation");
		_set_scale = (EventInMFVec2f)node.getEventIn("set_scale");
		_set_spine = (EventInMFVec3f)node.getEventIn("set_spine");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Extrusion;
	}

	private String createString()
	{
		int	i;
		String vrml_string = "Extrusion {";

		if(!_beginCap)
			vrml_string += " beginCap FALSE";

		if(!_ccw)
			vrml_string += " ccw FALSE";

		if(!_convex)
			vrml_string += " convex FALSE";

		if(_creaseAngle != 0);
			vrml_string += " creaseAngle " + _creaseAngle;

		if((_crossSection != null) && (_crossSection.length != 0))
		{
			vrml_string += " crossSection [";
			for(i = 0; i < _crossSection.length; i++)
				vrml_string += " " + _crossSection[i][0] +
							   " " + _crossSection[i][1];

			vrml_string += " ]";
		}

		if(!_endCap)
			vrml_string += " endCap FALSE";

		if((_orientation != null) && (_orientation.length != 0))
		{
			vrml_string += " orientation [";

			for(i = 0; i < _orientation.length; i++)
				vrml_string += " " + _orientation[i][0] +
							   " " + _orientation[i][1] +
							   " " + _orientation[i][2] +
							   " " + _orientation[i][3];

			vrml_string += " ]";
		}

		if((_scale != null) && (_scale.length != 0))
		{
			vrml_string += " scale [";
			for(i = 0; i < _scale.length; i++)
				vrml_string += " " + _scale[i][0] +
							   " " + _scale[i][1];

			vrml_string += " ]";
		}

		if(!_solid)
			vrml_string += " solid FALSE";

		if((_spine != null) && (_spine.length != 0))
		{
			vrml_string += " spine [";
			for(i = 0; i < _spine.length; i++)
				vrml_string += " " + _spine[i][0] +
							   " " + _spine[i][1] +
							   " " + _spine[i][2];

			vrml_string += " ]";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void finalize()
	{
		_crossSection = null;
		_orientation = null;
		_scale = null;
		_spine = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Extrusion {");

		if(!_beginCap)
			fp.println(buffer.toString() + " beginCap FALSE");

		if(!_ccw)
			fp.println(buffer.toString() + "  ccw FALSE");

		if(!_convex)
			fp.println(buffer.toString() + "  convex FALSE");

		if(_creaseAngle != 0);
			fp.println(buffer.toString() + "  creaseAngle " + _creaseAngle);

		if((_crossSection != null) && (_crossSection.length != 0))
		{
			fp.println(buffer.toString() + " crossSection [");

			for(i = 0; i < _crossSection.length; i++)
				fp.println(buffer.toString() +
						   " " + _crossSection[i][0] +
						   " " + _crossSection[i][1]);

			fp.println(buffer.toString() + " ]");
		}

		if(!_endCap)
				fp.println(buffer.toString() + " endCap FALSE");

		if((_orientation != null) && (_orientation.length != 0))
		{
			fp.println(buffer.toString() +" orientation [");

			for(i = 0; i < _orientation.length; i++)
				fp.println(buffer.toString() +
						   " " + _orientation[i][0] +
						   " " + _orientation[i][1] +
						   " " + _orientation[i][2] +
						   " " + _orientation[i][3]);

			fp.println(buffer.toString() +" ]");
		}

		if((_scale != null) && (_scale.length != 0))
		{
			fp.println(buffer.toString() + " scale [");

			for(i = 0; i < _scale.length; i++)
				fp.println(buffer.toString() +
						   " " + _scale[i][0] +
						   " " + _scale[i][1]);

			fp.println(buffer.toString() +" ]");
		}

		if(!_solid)
			fp.println(buffer.toString() + "  solid FALSE");

		if((_spine != null) && (_spine.length != 0))
		{
			fp.println(buffer.toString() +" spine [");

			for(i = 0; i < _spine.length; i++)
				fp.println(buffer.toString() +
						   " " + _spine[i][0] +
						   " " + _spine[i][1] +
						   " " + _spine[i][2]);

			fp.println(buffer.toString() +" ]");
		}

		fp.println(buffer.toString() + "}");
	}

	public boolean get_beginCap()
	{
		return(_beginCap);
	}

	public boolean get_ccw()
	{
		return(_ccw);
	}

	public boolean get_convex()
	{
		return(_convex);
	}

	public float get_creaseAngle()
	{
		return(_creaseAngle);
	}

	public void set_crossSection(float[][] sec)
	{
		_crossSection = sec;
		if(have_browser)
			_set_crossSection.setValue(sec);
	}

	public float[][] get_crossSection()
	{
		return(_crossSection);
	}

	public boolean get_endCap()
	{
		return(_endCap);
	}

	public void set_orientation(float[][] coord)
	{
		_orientation = coord;
		if(have_browser)
			_set_orientation.setValue(coord);
	}

	public float[][] get_orientation()
	{
		return(_orientation);
	}

	public void set_scale(float[][] scale)
	{
		_scale = scale;
		if(have_browser)
			_set_scale.setValue(scale);
	}

	public float[][] get_scale()
	{
		return(_scale);
	}

	public boolean get_solid()
	{
		return(_solid);
	}

	public void set_spine(float[][] spine)
	{
		_spine = spine;
		if(have_browser)
			_set_spine.setValue(spine);
	}

	public float[][] get_spine()
	{
		return(_spine);
	}
}
