// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// IndexedLineSet class.
// If you don't want to specify any of the MFint fields then just set the
// parameter to be null, and the system will handle it.

package vlc.vrml.geometry;

import java.io.PrintStream;
import vrml.external.*;
import vrml.external.field.*;
import vrml.external.exception.*;

public class IndexedLineSet extends VrmlObject
{
	private EventInMFInt32	_set_colorIndex;
	private EventInMFInt32	_set_coordIndex;
	private EventInSFNode	_set_color;
	private EventInSFNode	_set_coord;

	private Coordinate			_coord = null;
	private Color	_color = null;
	private int[]	_colorIndex = null;
	private int[]	_coordIndex = null;
	private boolean	_colorPerVertex = true;

	public IndexedLineSet()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("IndexedLineSet {}")[0];

		createFields();
	}

	public IndexedLineSet(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("IndexedLineSet {}")[0];

		createFields();
	}

	public IndexedLineSet(int[]   colorIndex,
						  int[]   coordIndex,
						  boolean colorPerVertex)
	{
		Browser b = Browser.getBrowser();

		_colorIndex = colorIndex;
		_coordIndex = coordIndex;
		_colorPerVertex = colorPerVertex;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	public IndexedLineSet(Browser b,
						  int[]   colorIndex,
						  int[]   coordIndex,
						  boolean colorPerVertex)
	{
		_colorIndex = colorIndex;
		_coordIndex = coordIndex;
		_colorPerVertex = colorPerVertex;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	private void createFields()
	{
		_set_color = (EventInSFNode)node.getEventIn("color");
		_set_coord = (EventInSFNode)node.getEventIn("coord");
		_set_colorIndex = (EventInMFInt32)node.getEventIn("set_colorIndex");
		_set_coordIndex = (EventInMFInt32)node.getEventIn("set_coordIndex");

		have_browser = true;
	}

	private String createString()
	{
		int	i;
		String vrml_string = "IndexedLineSet {";

		if(_colorIndex != null)
		{
			vrml_string += " colorIndex [";
			for(i = 0; i < _colorIndex.length; i++)
				vrml_string += " " + _colorIndex[i];

			vrml_string += " ]";
		}

		if(!_colorPerVertex)
			vrml_string += " colorPerVertex FALSE";

		if(_coordIndex != null)
		{
			vrml_string += " coordIndex [";
			for(i = 0; i < _coordIndex.length; i++)
				vrml_string += " " + _coordIndex[i];

			vrml_string += " ]";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		if(_color != null)
			_color.cleanup();

		if(_coord != null)
			_coord.cleanup();

		if(have_browser)
		{
			_set_color.setValue(null);
			_set_coord.setValue(null);
		}
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("IndexedLineSet {");

		if(_color != null)
			_color.writeToFile(fp, indent + 1);

		if(_coord != null)
			_coord.writeToFile(fp, indent + 1);

		if(_colorIndex != null)
		{
			fp.print(buffer.toString() + "  colorIndex ");

			if(_colorIndex.length == 1)
				fp.println(_colorIndex[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _colorIndex.length; i++)
					fp.println(buffer.toString() + "    " + _colorIndex[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(!_colorPerVertex)
			fp.println(buffer.toString() + "  colorPerVertex FALSE");

		if(_coordIndex != null)
		{
			fp.print(buffer.toString() + "  coordIndex ");

			if(_coordIndex.length == 1)
				fp.println(_coordIndex[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _coordIndex.length; i++)
					fp.println(buffer.toString() + "    " + _coordIndex[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		fp.println(buffer.toString() + "}");
	}

	public void set_color(Color color)
	{
		if(_color != null)
			_color.cleanup();

		_color = color;
		if(have_browser)
			_set_color.setValue(color.node);
	}

	public Color get_color()
	{
		return(_color);
	}

	public void set_coord(Coordinate coord)
	{
		if(_coord != null)
			_coord.cleanup();

		_coord = coord;
		if(have_browser)
			_set_coord.setValue(coord.node);
	}

	public Coordinate get_coord()
	{
		return(_coord);
	}

	public void set_colorIndex(int[] vals)
	{
		_colorIndex = vals;
		if(have_browser)
			_set_colorIndex.setValue(vals);
	}

	public int[] get_colorIndex()
	{
		return(_colorIndex);
	}

	public boolean get_colorPerVertex()
	{
		return(_colorPerVertex);
	}

	public void set_coordIndex(int[] vals)
	{
		_coordIndex = vals;
		if(have_browser)
			_set_coordIndex.setValue(vals);
	}

	public int[] get_coordIndex()
	{
		return(_coordIndex);
	}

}
