// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// PlaneSensor class
// We need to override the _offset and _set_offset methods and variables here
// because in the GeometrySensor they are a 4 value SFRotation rather than the
// required SFVec3f by the PlaneSensor.

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class PlaneSensor extends GeometrySensor
{
	private EventInSFVec2f	_set_maxPosition;
	private EventInSFVec2f	_set_minPosition;
	private EventInSFVec3f	_set_offset;
	private EventOutSFVec3f	_get_translation_changed;

	private float[]	_maxPosition = {-1, -1};
	private float[]	_minPosition = {0, 0};
	private float[]	_offset = { 0, 0, 0};

	public PlaneSensor()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("PlaneSensor {}")[0];

		createFields();
	}

	public PlaneSensor(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("PlaneSensor {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_enabled = (EventInSFBool)node.getEventIn("enabled");
		_set_maxPosition = (EventInSFVec2f)node.getEventIn("maxPosition");
		_set_minPosition = (EventInSFVec2f)node.getEventIn("minPosition");
		_set_offset = (EventInSFVec3f)node.getEventIn("offset");
		_set_autoOffset = (EventInSFBool)node.getEventIn("autoOffset");

		_get_isActive = (EventOutSFBool)node.getEventOut("isActive");
		_get_translation_changed = (EventOutSFVec3f)node.getEventOut("translation_changed");
		_get_trackPoint_changed = (EventOutSFVec3f)node.getEventOut("trackPoint_changed");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.PlaneSensor;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "PlaneSensor {");

		if(!_enabled)
			fp.println(buffer.toString() + "  enabled FALSE");

		if((_minPosition[0] != 0) || (_minPosition[1] != 0))
			fp.println(buffer.toString() + "  minPosition " +
			           _minPosition[0] + " " +
			           _minPosition[1]);
			
		if((_maxPosition[0] != -1) || (_maxPosition[1] != -1))
			fp.println(buffer.toString() + "  maxPosition " +
			           _maxPosition[0] + " " +
			           _maxPosition[1]);

		if((_offset[0] != 0) || (_offset[1] != 1) || (_offset[2] != 0))
		   	fp.println(buffer.toString() + "  offset " +
		   	           _offset[0] + " " +
		   	           _offset[1] + " " +
		   	           _offset[2]);
		   	           
		if(!_autoOffset)
			fp.println(buffer.toString() + "  autoOffset FALSE");

		fp.println(buffer.toString() + "}");
	}
	
	public void set_maxPosition(float[] pos)
	{
		_maxPosition = pos;
		if(have_browser)
			_set_maxPosition.setValue(pos);
	}

	public float[] get_maxPosition()
	{
		return(_maxPosition);
	}

	public void set_minPosition(float[] pos)
	{
		_minPosition = pos;
		if(have_browser)
			_set_minPosition.setValue(pos);
	}

	public float[] get_minPosition()
	{
		return(_minPosition);
	}

	public void set_offset(float[] offset)
	{
		_offset = offset;
		if(have_browser)
			_set_offset.setValue(offset);
	}

	public float[] get_offset()
	{
		return(_offset);
	}

	// returns null just in case someone calls this method which they shouldn't
	public float[] get_trackPoint_changed()
	{
		return null;
	}

	public float[] get_translation_changed()
	{
		return(_get_translation_changed.getValue());
	}
}