// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// PointLight class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class PointLight extends Light
{
	private EventInSFVec3f	_set_attenuation;
	private EventInSFVec3f	_set_location;
	private EventInSFFloat	_set_radius;

	private float[]	_attenuation = { 1, 0, 0};
	private float[]	_location = { 0, 0, 0};
	private float	_radius = 100;

	public PointLight()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("PointLight {}")[0];

		createFields();
	}

	public PointLight(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("PointLight {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_on = (EventInSFBool)node.getEventIn("on");
		_set_intensity = (EventInSFFloat)node.getEventIn("intensity");
		_set_ambientIntensity = (EventInSFFloat)node.getEventIn("ambientIntensity");
		_set_color = (EventInSFColor)node.getEventIn("color");
		_set_attenuation = (EventInSFVec3f)node.getEventIn("attenuation");
		_set_location = (EventInSFVec3f)node.getEventIn("location");
		_set_radius = (EventInSFFloat)node.getEventIn("radius");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.PointLight;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "PointLight {");

		if(!_on)
			fp.println(buffer.toString() + "  on FALSE");

		if(_ambientIntensity != 0)
			fp.println(buffer.toString() +
					   "  ambientIntensity " +
					   _ambientIntensity);

		if(_intensity != 0)
			fp.println(buffer.toString() +
			           "  intensity " +
			           _intensity);
			           
		if((_color[0] != 1) || (_color[1] != 1) || (_color[2] != 1))
			fp.println(buffer.toString() + "  color "
			           + _color[0] + " " +
			           + _color[1] + " " +
			           + _color[2]);
			           
		if((_attenuation[0] != 1) || (_attenuation[1] != 0) ||
		   (_attenuation[2] != 0))
			fp.println(buffer.toString() + "  attenuation "
			           + _attenuation[0] + " " +
			           + _attenuation[1] + " " +
			           + _attenuation[2]);
			           
		if((_location[0] != 1) || (_location[1] != 0) ||
		   (_location[2] != 0))
			fp.println(buffer.toString() + "  location "
			           + _location[0] + " " +
			           + _location[1] + " " +
			           + _location[2]);

		if(_radius != 100)
			fp.println(buffer.toString() + "  radius " + _radius);

		fp.println(buffer.toString() + "}");
	}
	
	public void set_attenuation(float[] val)
	{
		_attenuation = val;
		if(have_browser)
			_set_attenuation.setValue(val);
	}
	
	public float[] get_attenuation()
	{
		return(_attenuation);
	}

	public void set_location(float[] val)
	{
		_location = val;
		if(have_browser)
			_set_location.setValue(val);
	}
	
	public float[] get_location()
	{
		return(_location);
	}

	public void set_radius(float val)
	{
		_radius = val;
		if(have_browser)
			_set_radius.setValue(val);
	}
	
	public float get_radius()
	{
		return(_radius);
	}
	
}