// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// PointSet class.

package vlc.vrml.geometry;

import java.io.PrintStream;
import vrml.external.*;
import vrml.external.field.*;

public class PointSet extends VrmlObject
{
	private EventInSFNode _set_color;
	private EventInSFNode _set_coord;

	private Color _color = null;
	private Coordinate _coord = null;

	public PointSet()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("PointSet {}")[0];

		createFields();
	}

	public PointSet(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("PointSet {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_color = (EventInSFNode)node.getEventIn("color");
		_set_coord = (EventInSFNode)node.getEventIn("coord");
	}

	public void cleanup()
	{
		if(_color != null)
			_color.cleanup();

		if(_coord != null)
			_coord.cleanup();

		_color = null;
		_coord = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "color ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("PointSet {");

		if(_color != null)
			_color.writeToFile(fp, indent + 1);

		if(_coord != null)
			_coord.writeToFile(fp, indent + 1);

		fp.println(buffer.toString() + "}");
	}

	public void set_color(Color color)
	{
		if(_color != null)
			_color.cleanup();

		_color = color;
		if(have_browser)
			_set_color.setValue(color.node);
	}

	public Color get_color()
	{
		return(_color);
	}

	public void set_coord(Coordinate coord)
	{
		if(_coord != null)
			_coord.cleanup();

		_coord = coord;
		if(have_browser)
			_set_coord.setValue(coord.node);
	}

	public Coordinate get_coord()
	{
		return(_coord);
	}
}
