// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// ScalarInterpolator class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class ScalarInterpolator extends Interpolator
{
	private EventInMFFloat	_set_keyValue;

	private EventOutSFFloat	_get_value_changed;

	private float[]	_keyValue;

	public ScalarInterpolator()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("ScalarInterpolator {}")[0];

		createFields();
	}

	public ScalarInterpolator(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("ScalarInterpolator {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_keyValue = (EventInMFFloat)node.getEventIn("keyValue");

		_get_value_changed = (EventOutSFFloat)node.getEventOut("value_changed");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.ScalarInterpolator;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "ColorInterpolator {");

		if(_key != null)
		{
			fp.println(buffer.toString() + "  key [");
			
			for(i = 0; i < _key.length; i++)
				fp.println(buffer.toString() + "    " + _key[i]);

			fp.println(buffer.toString() + "  ]");
		}
		
		if(_keyValue != null)
		{
			fp.println(buffer.toString() + "  keyValue [");
			
			// now print the keyValues, 1 per line
			for(i = 0; i < _keyValue.length; i++)
				fp.println(buffer.toString() + "    " +
				           _keyValue[i]);

			fp.println(buffer.toString() + "  ]");
		}
		
		fp.println(buffer.toString() + "}");
	}
	
	public void set_keyValue(float[] scalar)
	{
		_keyValue = scalar;
		if(have_browser)
			_set_keyValue.setValue(scalar);
	}
	
	public float[] get_keyValue()
	{
		return(_keyValue);
	}
	
	public float get_value_changed()
	{
		return(_get_value_changed.getValue());
	}
}