// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Shape class

package vlc.vrml.external.geometry;

import java.io.*;
import java.awt.*;
import vrml.external.*;
import vrml.external.field.*;
import vrml.external.exception.*;

public class Shape extends ChildrenNode
{
	private EventInSFNode _set_appearance;
	private EventInSFNode _set_geometry;

	private Appearance _appearance = null;
	private Geometry _geometry = null;

	// constructor builds a copy of itself and is added to the scene
	// graph.
	public Shape()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Shape {}")[0];

		createFields();
	}

	public Shape(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Shape {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_appearance = (EventInSFNode)node.getEventIn("set_appearance");
		_set_geometry = (EventInSFNode)node.getEventIn("set_geometry");

		have_browser = true;
	}

	// memory cleanup and deallocation of screen objects
	public void cleanup()
	{
		_geometry.cleanup();
		_appearance.cleanup();

		if(have_browser)
		{
			_set_appearance.setValue(null);
			_set_geometry.setValue(null);
		}
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Shape {");

		// print the contents of the shape node
		if(_geometry != null)
			_geometry.writeToFile(fp, indent + 1);
		if(_appearance != null)
			_appearance.writeToFile(fp, indent + 1);

		fp.println(buffer.toString() + "}");
	}

	public void set_appearance(Appearance app)
	{
		if(_appearance != null)
			_appearance.cleanup();

		_appearance = app;
		if(have_browser)
			_set_appearance.setValue(app.node);
	}

	public Appearance get_appearance()
	{
		return(_appearance);
	}

	public void set_geometry(Geometry geom)
	{
		if(_geometry != null)
			_geometry.cleanup();
			
		_geometry = geom;

		if(have_browser)
			_set_geometry.setValue(geom.node);
	}

	public Geometry get_geometry()
	{
		return(_geometry);
	}

	public boolean hasGeometry()
	{
		return(_geometry != null);
	}

	public boolean hasAppearance()
	{
		return(_appearance != null);
	}

}
