// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// SphereSensor class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class SphereSensor extends GeometrySensor
{
	private EventInSFVec2f	_set_maxPosition;
	private EventInSFVec2f	_set_minPosition;
	private EventInSFRotation	_set_offset;
	
	private float[]	_maxPosition = {-1, -1};
	private float[]	_minPosition = {0, 0};
	private float[]	_offset = { 0, 0, 0};

	public SphereSensor()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("SphereSensor {}")[0];

		createFields();
	}
	
	public SphereSensor(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("SphereSensor {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_enabled = (EventInSFBool)node.getEventIn("enabled");
		_set_offset = (EventInSFRotation)node.getEventIn("offset");
		_set_autoOffset = (EventInSFBool)node.getEventIn("autoOffset");

		_get_isActive = (EventOutSFBool)node.getEventOut("isActive");
		_get_rotation_changed = (EventOutSFRotation)node.getEventOut("rotation_changed");
		_get_trackPoint_changed = (EventOutSFVec3f)node.getEventOut("trackPoint_changed");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.SphereSensor;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "SphereSensor {");

		if(!_enabled)
			fp.println(buffer.toString() + "  enabled FALSE");

		if((_offset[0] != 0) || (_offset[1] != 1) || (_offset[2] != 0))
		   	fp.println(buffer.toString() + "  offset " +
		   	           _offset[0] + " " +
		   	           _offset[1] + " " +
		   	           _offset[2]);
		   	           
		if(!_autoOffset)
			fp.println(buffer.toString() + "  autoOffset FALSE");

		fp.println(buffer.toString() + "}");
	}
}