// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// AudioClip class

package vlc.vrml.internal.geometry;

import java.io.PrintStream;
import vrml.*;
import vrml.field.*;
import vrml.node.*;

public class AudioClip extends VrmlObject implements AudioNode
{
	private MFString _set_url;
	private SFBool	_set_loop;
	private SFFloat	_set_pitch;
	private SFTime	_set_starttime;
	private SFTime	_set_stoptime;
	private SFString	_set_description;

	private ConstSFFloat	_get_duration_changed;
	private ConstSFBool	_get_isActive;

	private String[]	_url;
	private boolean	_loop = false;
	private float	_pitch = 1;
	private double	_startTime = 0;
	private double	_stopTime = 0;
	private String	_description = null;

	public AudioClip(Browser b)
	{
		if(b == null)
			return;
		try
		{
			node = (Node)b.createVrmlFromString("AudioClip {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	public void createFields()
	{
		_set_url = (MFString)node.getEventIn("url");
		_set_loop = (SFBool)node.getEventIn("loop");
		_set_pitch = (SFFloat)node.getEventIn("pitch");
		_set_starttime = (SFTime)node.getEventIn("startTime");
		_set_stoptime = (SFTime)node.getEventIn("stopTime");
		_set_description = (SFString)node.getEventIn("description");

		_get_duration_changed = (ConstSFFloat)node.getEventOut("duration_changed");
		_get_isActive = (ConstSFBool)node.getEventOut("isActive");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.AudioClip;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("AudioClip {");

		if(_url.length > 0)
		{
			fp.println(buffer.toString() + "  _url [");

			for(i = 0; i < _url.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _url[i] + "\"");
			}
			fp.println(buffer.toString() + " ]");
		}

		if(_loop)
			fp.println(buffer.toString() + "  loop TRUE");
		if(_pitch != 1)
			fp.println(buffer.toString() + "  pitch " + _pitch);
		if(_startTime != 0)
			fp.println(buffer.toString() + "  startTime " + _startTime);
		if(_stopTime != 0)
			fp.println(buffer.toString() + "  stopTime " + _stopTime);
		if(_description != null)
			fp.println(buffer.toString() + "  desciption \"" + _description + "\"");
	}

	public void set_url(String[] url)
	{
		_url = url;
		if(have_browser)
			_set_url.setValue(_url);
	}

	public String[] get_url()
	{
		return(_url);
	}

	public void set_loop(boolean val)
	{
		_loop = val;
		if(have_browser)
			_set_loop.setValue(val);
	}

	public boolean get_loop()
	{
		return(_loop);
	}

	public void set_pitch(float val)
	{
		_pitch = val;

		if(have_browser)
			_set_pitch.setValue(val);
	}

	public float get_pitch()
	{
		return(_pitch);
	}

	public void set_startTime(double val)
	{
		_startTime = val;

		if(have_browser)
			_set_starttime.setValue(val);
	}

	public double get_startTime()
	{
		return(_startTime);
	}

	public void set_stopTime(double val)
	{
		_stopTime = val;

		if(have_browser)
			_set_stoptime.setValue(val);
	}

	public double get_stopTime()
	{
		return(_stopTime);
	}

	public void set_description(String description)
	{
		_description = description;
		if(have_browser)
			_set_description.setValue(description);
	}

	public String get_description()
	{
		return(_description);
	}

	public float get_duration_changed()
	{
		return(_get_duration_changed.getValue());
	}

	public boolean get_isActive()
	{
		return(_get_isActive.getValue());
	}
}
