// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Background class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Background extends BindableNode
{
	private MFFloat	_set_groundAngle;
	private MFColor	_set_groundColor;
	private MFString	_set_backUrl;
	private MFString	_set_bottomUrl;
	private MFString	_set_frontUrl;
	private MFString	_set_leftUrl;
	private MFString	_set_rightUrl;
	private MFString	_set_topUrl;
	private MFColor	_set_skyColor;

	private float[]		_groundAngle;
	private float[][]	_groundColor;
	private float[][]	_skyColor;
	private String[]	_backUrl;
	private String[]	_bottomUrl;
	private String[]	_frontUrl;
	private String[]	_leftUrl;
	private String[]	_rightUrl;
	private String[]	_topUrl;

	public Background(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Background {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_bind = (SFBool)node.getEventIn("set_bind");
		_set_groundAngle = (MFFloat)node.getEventIn("groundAngle");
		_set_groundColor = (MFColor)node.getEventIn("groundColor");
		_set_skyColor = (MFColor)node.getEventIn("skyColor");
		_set_backUrl = (MFString)node.getEventIn("backUrl");
		_set_bottomUrl = (MFString)node.getEventIn("bottomUrl");
		_set_frontUrl = (MFString)node.getEventIn("frontUrl");
		_set_leftUrl = (MFString)node.getEventIn("leftUrl");
		_set_rightUrl = (MFString)node.getEventIn("rightUrl");
		_set_topUrl = (MFString)node.getEventIn("topUrl");

		_get_isBound = (ConstSFBool)node.getEventOut("isBound");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Background;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "Background {");

		if(_backUrl.length > 0)
		{
			fp.println(buffer.toString() + "  backUrl [");

			for(i = 0; i < _backUrl.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _backUrl[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}

		if(_bottomUrl.length > 0)
		{
			fp.println(buffer.toString() + "  bottomUrl [");

			for(i = 0; i < _bottomUrl.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _bottomUrl[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}

		if(_frontUrl.length > 0)
		{
			fp.println(buffer.toString() + "  frontUrl [");

			for(i = 0; i < _frontUrl.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _frontUrl[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}

		if(_leftUrl.length > 0)
		{
			fp.println(buffer.toString() + "  leftUrl [");

			for(i = 0; i < _leftUrl.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _leftUrl[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}

		if(_rightUrl.length > 0)
		{
			fp.println(buffer.toString() + "  rightUrl [");

			for(i = 0; i < _rightUrl.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _rightUrl[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}

		if(_topUrl.length > 0)
		{
			fp.println(buffer.toString() + "  topUrl [");

			for(i = 0; i < _topUrl.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _topUrl[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}

		fp.println(buffer.toString() + "}");
	}

	public void set_groundAngle(float[] val)
	{
		_groundAngle = val;
		if(have_browser)
			_set_groundAngle.setValue(val);
	}

	public float[] get_groundAngle()
	{
		return(_groundAngle);
	}

	public void set_groundColor(float[][] val)
	{
		_groundColor = val;
		if(have_browser)
			_set_groundColor.setValue(val);
	}

	public float[][] get_groundColor()
	{
		return(_groundColor);
	}

	public void set_skyColor(float[][] val)
	{
		_groundColor = val;
		if(have_browser)
			_set_skyColor.setValue(val);
	}

	public float[][] get_skyColor()
	{
		return(_skyColor);
	}

	public void set_backUrl(String[] val)
	{
		_backUrl = val;
		if(have_browser)
			_set_backUrl.setValue(val);
	}

	public String[] get_backUrl()
	{
		return(_backUrl);
	}

	public void set_bottomUrl(String[] val)
	{
		_bottomUrl = val;
		if(have_browser)
			_set_bottomUrl.setValue(val);
	}

	public String[] get_bottomUrl()
	{
		return(_bottomUrl);
	}

	public void set_frontUrl(String[] val)
	{
		_frontUrl = val;
		if(have_browser)
			_set_frontUrl.setValue(val);
	}

	public String[] get_frontUrl()
	{
		return(_frontUrl);
	}

	public void set_leftUrl(String[] val)
	{
		_leftUrl = val;
		if(have_browser)
			_set_leftUrl.setValue(val);
	}

	public String[] get_leftUrl()
	{
		return(_leftUrl);
	}

	public void set_rightUrl(String[] val)
	{
		_rightUrl = val;
		if(have_browser)
			_set_rightUrl.setValue(val);
	}

	public String[] get_rightUrl()
	{
		return(_rightUrl);
	}

	public void set_topUrl(String[] val)
	{
		_topUrl = val;
		if(have_browser)
			_set_topUrl.setValue(val);
	}

	public String[] get_topUrl()
	{
		return(_topUrl);
	}

}