// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Cone class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Cone extends Geometry
{
	private boolean	isDefault = true;

	private float _radius = 1;
	private float _height = 2;
	private boolean _side = true;
	private boolean _bottom = true;

	public Cone(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Cone {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	public Cone(Browser b,
				float radius,
				float height,
				boolean showSide,
				boolean showBottom)
	{
		_radius = radius;
		_height = height;
		_side = showSide;
		_bottom = showBottom;

		isDefault = false;

		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString(createString())[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Cone;
	}

	private String createString()
	{
		String vrml_string = "Cone {";

		if(_radius != 1)
			vrml_string += " bottomRadius " + _radius;

		if(_height != 2)
			vrml_string += " height " + _height;

		if(!_side)
			vrml_string += " side FALSE";

		if(!_bottom)
			vrml_string += "bottom FALSE";

		vrml_string += " }";

		return(vrml_string);
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "geometry ");
		if(name != null)
			fp.print("DEF " + name + " ");


		if(isDefault)
		{
			fp.println("Cone {}");
			return;
		}

		fp.println("Cone {");

		if(_radius != 1)
			fp.println(buffer.toString() +
					   " bottomRadius " +
					   _radius);

		if(_height != 2)
			fp.println(buffer.toString() +
					   " height " +
					   _height);
		if(!_side)
			fp.println(buffer.toString() + " side FALSE");

		if(!_bottom)
			fp.println(buffer.toString() + " bottom FALSE ");

		fp.println(buffer.toString() + "}");
	}

	public float get_bottomRadius()
	{
		return(_radius);
	}

	public float get_height()
	{
		return(_height);
	}

	public boolean get_side()
	{
		return(_side);
	}

	public boolean get_bottom()
	{
		return(_bottom);
	}
}
