// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 25: Late Night VRML 2.0 and Java
//
// Group class

package vlc.vrml.internal.geometry;

import java.io.*;
import java.util.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Group extends GroupingNode
{
	public Group(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Group {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	public Group(Browser b, float[] bboxCenter, float[] bboxSize)
	{
		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString(createString())[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_addChildren = (MFNode)node.getEventIn("addChildren");
		_removeChildren = (MFNode)node.getEventIn("removeChildren");
		_set_children = (MFNode)node.getEventIn("children");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Group;
	}

	private String createString()
	{
		String vrml_string = "Group { ";

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
		{
			vrml_string += "bboxCenter " +
							_bboxCenter[0] + " " +
							_bboxCenter[1] + " " +
							_bboxCenter[2] + " ";
		}

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
		{
			vrml_string += "bboxCenter " +
							_bboxSize[0] + " " +
							_bboxSize[1] + " " +
							_bboxSize[2] + " ";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		int	i;
		
		for(i = 0; i < _children.size(); i++)
			removeChildren((ChildrenNode)_children.elementAt(i));
			
		// force the clean up of internally allocated objects
		_children = null;
		_bboxCenter = null;
		_bboxSize = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Group {");

		// print the contents of the node

		if(num_children != 0)
		{
			Enumeration e = _children.elements();

			fp.print(buffer.toString() + "  children " );
			if(num_children != 1)
				fp.println("[");

			for(;e.hasMoreElements();)
				((VrmlObject)(e.nextElement())).writeToFile(fp, indent + 2);

			if(num_children != 1)
				fp.println(buffer.toString() + "  ]");
		}

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
			fp.println(buffer.toString() + " bboxCenter " +
						_bboxCenter[0] + " " +
						_bboxCenter[1] + " " +
						_bboxCenter[2]);

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
			fp.println(buffer.toString() + " bboxSize " +
						_bboxSize[0] + " " +
						_bboxSize[1] + " " +
						_bboxSize[2]);

		fp.println(buffer.toString() + "}");
	}
}
