// VRML Generator
// (c) Justin Couch 1996
//
// Chapter 25: Late Night VRML 2.0 and Java
//
// GroupingNode abstract class. Base class for all the Grouping Nodes and
// SpecialGroups

package vlc.vrml.internal.geometry;

import java.io.PrintStream;
import java.util.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public abstract class GroupingNode extends ChildrenNode
{
	protected MFNode		_addChildren;
	protected MFNode		_removeChildren;
	protected MFNode		_set_children;

	protected int			num_children = 0;
	protected Vector		_children = new Vector();
	protected float[]		_bboxCenter = {0, 0, 0};
	protected float[]		_bboxSize = {-1, -1, -1};


	// make this a protected constructor so that you cannot instantiate a
	// copy of this node directly.
	protected GroupingNode()
	{
	}

	public void addChildren(ChildrenNode child)
	{
		Node[] n = new Node[1];
		n[0] = child.node;

		_children.addElement(child);
		if(have_browser)
			_addChildren.setValue(n);
		num_children++;
	}

	public void removeChildren(ChildrenNode child)
	{
		Node[] n = new Node[1];
		n[0] = child.node;

		child.cleanup();

		_children.removeElement(child);
		if(have_browser)
			_addChildren.setValue(n);
		num_children--;
	}

	public void set_children(ChildrenNode[] child_list)
	{
		int	i;
		int num = 0;
		Node[] 	node_list = null;

		if(child_list != null)
		{
			num = child_list.length;
			// assign the new arrays.
			_children = new Vector(num);
			node_list = new Node[num];
		}
		// copy the values across.
		for(i = 0; i < num; i++)
		{
			_children.addElement(child_list[i]);
			node_list[i] = child_list[i].node;
		}

		// set the values in the VRML scene
		if(have_browser)
			_set_children.setValue(node_list);
	}

	public ChildrenNode[] get_children()
	{
		int	i;
		int	size = _children.size();
		ChildrenNode[] node_list = new ChildrenNode[size];

		for(i = 0; i < size; i++)
		{
			node_list[i] = (ChildrenNode)_children.elementAt(i);
		}

		return(node_list);
	}

	public float[] get_bboxCenter()
	{
		return(_bboxCenter);
	}

	public float[] get_bboxSize()
	{
		return(_bboxSize);
	}
}