// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// IndexedFaceSet class.
// If you don't want to specify any of the MFint fields then just set the
// parameter to be null, and the system will handle it.

package vlc.vrml.internal.geometry;

import java.io.PrintStream;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class IndexedFaceSet extends VrmlObject
{
	private MFInt32	_set_colorIndex;
	private MFInt32	_set_coordIndex;
	private MFInt32	_set_normalIndex;
	private MFInt32	_set_texCoordIndex;
	private SFNode	_set_color;
	private SFNode	_set_coord;
	private SFNode	_set_normal;
	private SFNode	_set_texCoord;

	private Coordinate			_coord = null;
	private TextureCoordinate	_texCoord = null;
	private Color	_color = null;
	private Normal	_normal = null;
	private boolean	_ccw = true;
	private int[]	_colorIndex = null;
	private int[]	_coordIndex = null;
	private boolean	_colorPerVertex = true;
	private boolean	_convex = true;
	private float	_creaseAngle = 0;
	private int[]	_normalIndex = null;
	private boolean	_normalPerVertex = true;
	private boolean	_solid = true;
	private int[]	_texCoordIndex = null;

	public IndexedFaceSet(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("IndexedFaceSet {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	public IndexedFaceSet(Browser b,
						  boolean ccw,
						  int[]   colorIndex,
						  int[]   coordIndex,
						  boolean colorPerVertex,
						  boolean convex,
						  float   creaseAngle,
						  int[]   normalIndex,
						  boolean normalPerVertex,
						  boolean solid,
						  int[]   texCoordIndex)
	{
		_ccw = ccw;
		_colorIndex = colorIndex;
		_coordIndex = coordIndex;
		_colorPerVertex = colorPerVertex;
		_convex = convex;
		_creaseAngle = creaseAngle;
		_normalIndex = normalIndex;
		_normalPerVertex = normalPerVertex;
		_solid = solid;
		_texCoordIndex = texCoordIndex;

		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString(createString())[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_color = (SFNode)node.getEventIn("color");
		_set_coord = (SFNode)node.getEventIn("coord");
		_set_normal = (SFNode)node.getEventIn("normal");
		_set_texCoord = (SFNode)node.getEventIn("texCoord");
		_set_colorIndex = (MFInt32)node.getEventIn("set_colorIndex");
		_set_coordIndex = (MFInt32)node.getEventIn("set_coordIndex");
		_set_normalIndex = (MFInt32)node.getEventIn("set_normalIndex");
		_set_texCoordIndex = (MFInt32)node.getEventIn("set_texCoordIndex");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.IndexedFaceSet;
	}

	private String createString()
	{
		int	i;
		String vrml_string = "IndexedFaceSet {";

		if(!_ccw)
			vrml_string += " ccw FALSE";

		if(_colorIndex != null)
		{
			vrml_string += " colorIndex [";
			for(i = 0; i < _colorIndex.length; i++)
				vrml_string += " " + _colorIndex[i];

			vrml_string += " ]";
		}

		if(!_colorPerVertex)
			vrml_string += " colorPerVertex FALSE";

		if(!_convex)
			vrml_string += " convex FALSE";

		if(_coordIndex != null)
		{
			vrml_string += " coordIndex [";
			for(i = 0; i < _coordIndex.length; i++)
				vrml_string += " " + _coordIndex[i];

			vrml_string += " ]";
		}

		if(_creaseAngle != 0);
			vrml_string += " creaseAngle " + _creaseAngle;

		if(_normalIndex != null)
		{
			vrml_string += " normalIndex [";
			for(i = 0; i < _normalIndex.length; i++)
				vrml_string += " " + _normalIndex[i];

			vrml_string += " ]";
		}

		if(!_normalPerVertex)
			vrml_string += " normalPerVertex FALSE";

		if(!_solid)
			vrml_string += " solid FALSE";

		if(_texCoordIndex != null)
		{
			vrml_string += " texCoordIndex [";
			for(i = 0; i < _texCoordIndex.length; i++)
				vrml_string += " " + _texCoordIndex[i];

			vrml_string += " ]";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		if(_color != null)
			_color.cleanup();

		if(_coord != null)
			_coord.cleanup();

		if(_normal != null)
			_normal.cleanup();

		if(_texCoord != null)
			_texCoord.cleanup();

		if(have_browser)
		{
			_set_color.setValue((SFNode)null);
			_set_coord.setValue((SFNode)null);
			_set_normal.setValue((SFNode)null);
			_set_texCoord.setValue((SFNode)null);
		}
	}


	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("IndexedFaceSet {");

		if(!_ccw)
			fp.println(buffer.toString() + "  ccw FALSE");

		if(_colorIndex != null)
		{
			fp.print(buffer.toString() + "  colorIndex ");

			if(_colorIndex.length == 1)
				fp.println(_colorIndex[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _colorIndex.length; i++)
					fp.println(buffer.toString() + "    " + _colorIndex[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(!_colorPerVertex)
			fp.println(buffer.toString() + "  colorPerVertex FALSE");

		if(!_convex)
			fp.println(buffer.toString() + "  convex FALSE");

		if(_coordIndex != null)
		{
			fp.print(buffer.toString() + "  coordIndex ");

			if(_coordIndex.length == 1)
				fp.println(_coordIndex[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _coordIndex.length; i++)
					fp.println(buffer.toString() + "    " + _coordIndex[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(_creaseAngle != 0);
			fp.println(buffer.toString() + "  creaseAngle " + _creaseAngle);

		if(_normalIndex != null)
		{
			fp.print(buffer.toString() + "  normalIndex ");

			if(_normalIndex.length == 1)
				fp.println(_normalIndex[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _normalIndex.length; i++)
					fp.println(buffer.toString() + "    " + _normalIndex[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(!_normalPerVertex)
			fp.println(buffer.toString() + "  normalPerVertex FALSE");

		if(!_solid)
			fp.println(buffer.toString() + "  solid FALSE");

		if(_texCoordIndex != null)
		{
			fp.print(buffer.toString() + "  texCoordIndex ");

			if(_texCoordIndex.length == 1)
				fp.println(_texCoordIndex[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _texCoordIndex.length; i++)
					fp.println(buffer.toString() + "    " + _texCoordIndex[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(_color != null)
			_color.writeToFile(fp, indent + 1);

		if(_coord != null)
			_coord.writeToFile(fp, indent + 1);

		if(_normal != null)
			_normal.writeToFile(fp, indent + 1);

		if(_texCoord != null)
			_texCoord.writeToFile(fp, indent + 1);

		fp.println(buffer.toString() + "}");
	}

	public void set_color(Color color)
	{
		if(_color != null)
			_color.cleanup();

		_color = color;
		if(have_browser)
		{
			if(color != null)
				_set_color.setValue(color.node);
			else
				_set_color.setValue((SFNode)null);
		}

	}

	public Color get_color()
	{
		return(_color);
	}

	public void set_coord(Coordinate coord)
	{
		if(_coord != null)
			_coord.cleanup();

		_coord = coord;
		if(have_browser)
		{
			if(coord != null)
				_set_coord.setValue(coord.node);
			else
				_set_coord.setValue((SFNode)null);
		}

	}

	public Coordinate get_coord()
	{
		return(_coord);
	}

	public void set_normal(Normal normal)
	{
		if(_normal != null)
			_normal.cleanup();

		_normal = normal;
		if(have_browser)
		{
			if(normal != null)
				_set_normal.setValue(normal.node);
			else
				_set_normal.setValue((SFNode)null);
		}

	}

	public Normal get_normal()
	{
		return(_normal);
	}

	public void set_texCoord(TextureCoordinate texCoord)
	{
		if(_texCoord != null)
			_texCoord.cleanup();

		_texCoord = texCoord;
		if(have_browser)
		{
			if(texCoord != null)
				_set_texCoord.setValue(texCoord.node);
			else
				_set_texCoord.setValue((SFNode)null);
		}

	}

	public TextureCoordinate get_texCoord()
	{
		return(_texCoord);
	}

	public boolean get_ccw()
	{
		return(_ccw);
	}

	public void set_colorIndex(int[] vals)
	{
		_colorIndex = vals;
		if(have_browser)
			_set_colorIndex.setValue(vals);
	}

	public int[] get_colorIndex()
	{
		return(_colorIndex);
	}

	public boolean get_colorPerVertex()
	{
		return(_colorPerVertex);
	}

	public boolean get_convex()
	{
		return(_convex);
	}

	public void set_coordIndex(int[] vals)
	{
		_coordIndex = vals;
		if(have_browser)
			_set_coordIndex.setValue(vals);
	}

	public int[] get_coordIndex()
	{
		return(_coordIndex);
	}

	public float get_creaseAngle()
	{
		return(_creaseAngle);
	}

	public void set_normalIndex(int[] vals)
	{
		_normalIndex = vals;
		if(have_browser)
			_set_normalIndex.setValue(vals);
	}

	public int[] get_normalIndex()
	{
		return(_normalIndex);
	}

	public boolean get_normalPerVertex()
	{
		return(_normalPerVertex);
	}

	public boolean get_solid()
	{
		return(_solid);
	}

	public void set_texCoordIndex(int[] vals)
	{
		_texCoordIndex = vals;
		if(have_browser)
			_set_texCoordIndex.setValue(vals);
	}

	public int[] get_texCoordIndex()
	{
		return(_texCoordIndex);
	}


}
