// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Collision class
// The class does not support proxy nodes yet. Still trying to work this out.

package vlc.vrml.internal.geometry;

import java.io.*;
import java.util.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class LOD extends SpecialGroup
{
	private MFNode	_set_level;

	private int	num_level;
	private float[]	_center = {0, 0, 0};
	private float[]	_range	= null;
	private Vector	_level = new Vector();

	public LOD(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("LOD {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	public LOD(Browser b, float[] center, float[] range)
	{
		if((center != null) && (center.length > 2))
			_center = center;

		if((range != null) && (range.length > 0))
			_range = range;

		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString(createString())[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}


		createFields();
	}

	private String createString()
	{
		int	i;
		String	vrml_string = "LOD {";

		if((_center!= null) &&
		   ((_center[0] != 0) || (_center[1] != 0) || (_center[2] != 0)))
		   vrml_string += " center " +
						  _center[0] + " " +
						  _center[1] + " " +
						  _center[2];

		if(_range != null)
		{
			vrml_string += " range [ ";

			for(i = 0; i < _range.length; i++)
				vrml_string += _range[i] + " ";

			vrml_string += " ]";
		}

		vrml_string += "}";

		return(vrml_string);
	}

	private void createFields()
	{
		_set_level = (MFNode)node.getEventIn("level");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.LOD;
	}

	public void cleanup()
	{
		// force the clean up of internally allocated objects
		int	i;

		for(i = 0; i < _level.size(); i++)
			removeLevel((ChildrenNode)_level.elementAt(i));

		_level = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("LOD {");

		// print the contents of the node

		if(_center != null)
			fp.println(buffer.toString() + "  center " +
					   _center[0] + " " +
					   _center[1] + " " +
					   _center[2]);

		if(_range != null)
		{
			fp.print(buffer.toString() + "  range " );

			if(_range.length == 1)
				fp.println(_range[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _range.length; i++)
					fp.println(buffer.toString() + "  " + _range[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(_level.size() != 0)
		{
			Enumeration e = _level.elements();

			fp.print(buffer.toString() + "  level " );

			if(_level.size() != 1)
				fp.println("[");

			while(e.hasMoreElements())
				((VrmlObject)(e.nextElement())).writeToFile(fp, indent + 2);

			if(_level.size() != 1)
				fp.println(buffer.toString() + "  ]");
		}
	}

	public void addLevel(ChildrenNode child)
	{
		int		i;
		Node[]	node_list;

		_level.addElement(child);

		if(have_browser)
		{
			node_list = new Node[_level.size()];

			// copy the values across.
			for(i = 0; i < _level.size(); i++)
				node_list[i] = ((VrmlObject)_level.elementAt(i)).node;

			_set_level.setValue(node_list);
		}

		num_level++;
	}

	public void removeLevel(ChildrenNode child)
	{
		int		i;
		Node[] 	node_list;

		_level.removeElement(child);

		num_level--;

		if(have_browser)
		{
			if(num_level == 0)
			{
				_set_level.setValue((MFNode)null);
				return;
			}

			node_list = new Node[_level.size()];

			// copy the values across.
			for(i = 0; i < _level.size(); i++)
				node_list[i] = ((VrmlObject)_level.elementAt(i)).node;

			_set_level.setValue(node_list);
		}

	}

	public void set_level(ChildrenNode[] child_list)
	{
		int	i;
		int num = 0;
		Node[] 	node_list = null;

		if(child_list != null)
		{
			num = child_list.length;
			// assign the new arrays.
			_level = new Vector(num);
			node_list = new Node[num];
		}

		// copy the values across.
		for(i = 0; i < num; i++)
		{
			_level.addElement(child_list[i]);
			node_list[i] = child_list[i].node;
		}

		// set the values in the VRML scene
		if(have_browser)
			_set_level.setValue(node_list);
	}

	public ChildrenNode[] get_level()
	{
 		int	i;
		int	size = _level.size();
		ChildrenNode[] node_list = new ChildrenNode[size];

		for(i = 0; i < size; i++)
		{
			node_list[i] = (ChildrenNode)_level.elementAt(i);
		}
		
		return(node_list);
	}

	public float[] get_range()
	{
		return(_range);
	}

	public float[] get_center()
	{
		return(_center);
	}
}
