// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 25: Late Night VRML 2.0 and Java
//
// NavigationInfo class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class NavigationInfo extends BindableNode
{
	private MFFloat	_set_avatarSize;
	private SFBool	_set_headlight;
	private SFFloat	_set_speed;
	private MFString	_set_type;
	private SFFloat	_set_visibilityLimit;

	private float[]	_avatarSize = { 0.25f, 1.6f, 0.75f};
	private boolean _headlight = true;
	private float	_speed = 1;
	private String[]	_type	= {"WALK"};
	private float	_visibilityLimit =  0;

	public NavigationInfo(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("NavigationInfo {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}
		
		createFields();
	}

	private void createFields()
	{
		_set_bind = (SFBool)node.getEventIn("set_bind");
		_set_avatarSize = (MFFloat)node.getEventIn("avatarSize");
		_set_headlight = (SFBool)node.getEventIn("headlight");
		_set_speed = (SFFloat)node.getEventIn("speed");
		_set_type = (MFString)node.getEventIn("type");
		_set_visibilityLimit = (SFFloat)node.getEventIn("visibilityLimit");

		_get_isBound = (ConstSFBool)node.getEventOut("isBound");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.NavigationInfo;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "NavigationInfo {");

		if((_avatarSize.length != 3) || (_avatarSize[0] != 0.25f) ||
		   (_avatarSize[1] != 1.6f) || (_avatarSize[2] != 0.72f))
		{
			fp.println(buffer.toString() + "  avatarSize [");

			for(i = 0; i < _avatarSize.length;  i++)
				fp.print(_avatarSize[i] + " ");

			fp.println("]");
		}

		if(!_headlight)
			fp.println(buffer.toString() + "  headlight TRUE");

		if(_speed != 1)
			fp.println(buffer.toString() + "  speed " + _speed);

		if((_type.length > 1) || (!_type[0].equals("WALK")))
		{
				fp.print(buffer.toString() + "  type ");

				if(_type.length == 1)
					fp.println(_type[0]);
				else
				{
					fp.println(buffer.toString() + "[");

					for(i = 0; i < _type.length; i++)
						fp.println(buffer.toString() + "    " + _type[i]);

					fp.println(buffer.toString() + "  ]");
				}
		}

		if(_visibilityLimit != 0)
			fp.println(buffer.toString() + "  visibilityLimit " + _visibilityLimit);

		fp.println(buffer.toString() + "}");
	}

	public void set_avatarSize(float[] sizes)
	{
		_avatarSize = sizes;
		if(have_browser)
			_set_avatarSize.setValue(sizes);
	}

	public float[] get_avatarSize()
	{
		return(_avatarSize);
	}

	public void set_headlight(boolean on)
	{
		_headlight = on;
		if(have_browser)
			_set_headlight.setValue(on);
	}

	public boolean get_headlight()
	{
		return(_headlight);
	}

	public void set_speed(float speed)
	{
		_speed = speed;
		if(have_browser)
			_set_speed.setValue(speed);
	}

	public float get_speed()
	{
		return(_speed);
	}

	public void set_type(String[] types)
	{
		_type = types;
		if(have_browser)
			_set_type.setValue(types);
	}

	public String[] get_type()
	{
		return(_type);
	}

	public void set_visibilityLimit(float limit)
	{
		_visibilityLimit = limit;
		if(have_browser)
			_set_visibilityLimit.setValue(limit);
	}

	public float get_visibilityLimit()
	{
		return(_visibilityLimit);
	}
}
