// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Shape class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Shape extends ChildrenNode
{
	private SFNode _set_appearance;
	private SFNode _set_geometry;

	private Appearance _appearance = null;
	private Geometry _geometry = null;

	public Shape(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Shape {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_appearance = (SFNode)node.getEventIn("set_appearance");
		_set_geometry = (SFNode)node.getEventIn("set_geometry");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Shape;
	}

	// memory cleanup and deallocation of screen objects
	public void cleanup()
	{
		_geometry.cleanup();
		_appearance.cleanup();

		if(have_browser)
		{
			_set_appearance.setValue((SFNode)null);
			_set_geometry.setValue((SFNode)null);
		}
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Shape {");

		// print the contents of the shape node
		if(_geometry != null)
			_geometry.writeToFile(fp, indent + 1);
		if(_appearance != null)
			_appearance.writeToFile(fp, indent + 1);

		fp.println(buffer.toString() + "}");
	}

	public void set_appearance(Appearance app)
	{
		if(_appearance != null)
			_appearance.cleanup();

		_appearance = app;

		if(have_browser)
		{
			if(app != null)
				_set_appearance.setValue(app.node);
			else
				_set_appearance.setValue((SFNode)null);
		}
	}

	public Appearance get_appearance()
	{
		return(_appearance);
	}

	public void set_geometry(Geometry geom)
	{
		if(_geometry != null)
			_geometry.cleanup();

		_geometry = geom;

		System.out.println("about to send event to the browser");

		if(have_browser)
		{
			if(geom != null)
				_set_geometry.setValue(geom.node);
			else
				_set_geometry.setValue((SFNode)null);
		}
	}

	public Geometry get_geometry()
	{
		return(_geometry);
	}

	public boolean hasGeometry()
	{
		return(_geometry != null);
	}

	public boolean hasAppearance()
	{
		return(_appearance != null);
	}

}
