// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Text class.

package vlc.vrml.internal.geometry;

import java.io.PrintStream;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Text extends Geometry
{
	private SFNode	_set_fontStyle;
	private MFString _set_string;
	private MFFloat	_set_length;
	private SFFloat	_set_maxExtent;

	private FontStyle	_fontStyle = null;
	private String[]	_string;
	private float[]		_length;
	private float		_maxExtent = 0;

	public Text(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Text {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_fontStyle = (SFNode)node.getEventIn("fontStyle");
		_set_string = (MFString)node.getEventIn("string");
		_set_length = (MFFloat)node.getEventIn("length");
		_set_maxExtent = (SFFloat)node.getEventIn("maxExtent");

		have_browser = true;
	}

	public void cleanup()
	{
		if(_fontStyle != null)
			_fontStyle.cleanup();
		_fontStyle = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Text {");

		if(_fontStyle != null)
			_fontStyle.writeToFile(fp, indent + 1);

		if(_string.length > 0)
		{
			fp.print(buffer.toString() + "  string ");

			if(_string.length == 1)
				fp.println(_string[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _string.length; i++)
					fp.println(buffer.toString() +  "  \"" +
							   _string[i] + "\"");

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(_length.length > 0)
		{
			fp.print(buffer.toString() + "  length ");

			if(_length.length == 1)
				fp.println(_length[0]);
			else
			{
				fp.println("[");

				for(i = 0; i < _length.length; i++)
					fp.println(buffer.toString() + _length[i]);

				fp.println(buffer.toString() + "  ]");
			}
		}

		if(_maxExtent != 0)
			fp.println(buffer.toString() + "  maxExtent " + _maxExtent);

		fp.println(buffer.toString() + "}");
	}

	public void set_fontStyle(FontStyle fontStyle)
	{
		if(_fontStyle != null)
			_fontStyle.cleanup();

		_fontStyle = fontStyle;
		if(have_browser)
			_set_fontStyle.setValue(fontStyle.node);
	}

	public FontStyle get_fontStyle()
	{
		return(_fontStyle);
	}

	public void set_string(String[] string)
	{
		_string = string;
		if(have_browser)
			_set_string.setValue(string);
	}

	public String[] get_string()
	{
		return(_string);
	}

	public void set_length(float[] length)
	{
		_length = length;
		if(have_browser)
			_set_length.setValue(length);
	}

	public float[] get_length()
	{
		return(_length);
	}

	public void set_maxExtent(float maxExtent)
	{
		_maxExtent = maxExtent;
		if(have_browser)
			_set_maxExtent.setValue(maxExtent);
	}

	public float get_maxExtent()
	{
		return(_maxExtent);
	}

}
