// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 25: Late Night VRML 2.0 and Java
//
// TextureTransform class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class TextureTransform extends VrmlObject
{
	SFVec2f	_set_middle;
	SFVec2f	_set_scale;
	SFVec2f	_set_trans;
	SFFloat	_set_rotate;

	float[] center = {0, 0};
	float[] scale = {1, 1};
	float[] translation = {0, 0};
	float	rotation = 0;

	boolean center_changed = false;
	boolean scale_changed = false;
	boolean translation_changed = false;

	public TextureTransform(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("TextureTransform {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_middle = (SFVec2f)node.getEventIn("center");
		_set_scale = (SFVec2f)node.getEventIn("scale");
		_set_trans = (SFVec2f)node.getEventIn("translation");
		_set_rotate = (SFFloat)node.getEventIn("rotation");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.TextureTransform;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "textureTransform ");
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("TextureTransform {");

		// print the contents of the shape node
		if(center_changed)
			fp.println(buffer.toString() + " center " +
				   center[0] + " " +
				   center[1]);
		if(scale_changed)
			fp.println(buffer.toString() + " scale " +
				   scale[0] + " " +
				   scale[1]);
		if(translation_changed)
			fp.println(buffer.toString() + " translation " +
				   translation[0] + " " +
				   translation[1]);
		if(rotation != 0)
			fp.println(buffer.toString() + " rotation " + rotation);

		fp.println(buffer.toString() + "}");
	}

	public void set_center(float x, float y)
	{
		center[0] = x;
		center[1] = y;

		if(have_browser);
			_set_middle.setValue(center);

		if((x == 0) && (y == 0))
			center_changed = false;
		else
			center_changed = true;
	}

	public float[] get_center()
	{
		return(center);
	}

	public void set_scale(float x, float y)
	{
		scale[0] = x;
		scale[1] = y;

		if(have_browser);
			_set_scale.setValue(scale);

		if((x == 0) && (y == 0))
			scale_changed = false;
		else
			scale_changed = true;
	}

	public float[] get_scale()
	{
		return(scale);
	}

	public void set_translation(float x, float y)
	{
		translation[0] = x;
		translation[1] = y;

		if(have_browser);
			_set_trans.setValue(scale);

		if((x == 0) && (y == 0))
			translation_changed = false;
		else
			translation_changed = true;
	}

	public float[] get_translation()
	{
		return(translation);
	}

	public void set_rotation(float val)
	{
		rotation = val;

		if(have_browser);
			_set_rotate.setValue(rotation);
	}

	public float get_rotation()
	{
		return(rotation);
	}
}
