// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 25: Late Night VRML 2.0 and Java
//
// VisibilitySensor class
// We need to override the _offset and _set_offset methods and variables here
// because in the GeometrySensor they are a 4 value SFRotation rather than the
// required SFVec3f by the VisibilitySensor.

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class VisibilitySensor extends Sensor
{
	private SFVec3f	_set_center;
	private SFVec3f	_set_size;

	private ConstSFTime	_get_enterTime;
	private ConstSFTime	_get_exitTime;

	private float[]	_center = {0, 0, 0};
	private float[]	_size = {0, 0, 0};

	public VisibilitySensor(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("VisibilitySensor {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_enabled = (SFBool)node.getEventIn("enabled");
		_set_center = (SFVec3f)node.getEventIn("center");
		_set_size = (SFVec3f)node.getEventIn("size");

		_get_isActive = (ConstSFBool)node.getEventOut("isActive");
		_get_enterTime = (ConstSFTime)node.getEventOut("enterTime");
		_get_exitTime = (ConstSFTime)node.getEventOut("exitTime");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.VisibilitySensor;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "VisibilitySensor {");

		if(!_enabled)
			fp.println(buffer.toString() + "  enabled FALSE");

		if((_center[0] != 0) || (_center[1] != 0) || (_center[2] != 0))
			fp.println(buffer.toString() + "  center " +
			           _center[0] + " " +
			           _center[1] + " " +
			           _center[2]);
			
		if((_size[0] != 0) || (_size[1] != 0) || (_size[2] != 0))
			fp.println(buffer.toString() + "  size " +
			           _size[0] + " " +
			           _size[1] + " " +
			           _size[2]);

		fp.println(buffer.toString() + "}");
	}
	
	public void set_size(float[] pos)
	{
		_size = pos;
		if(have_browser)
			_set_size.setValue(pos);
	}

	public float[] get_size()
	{
		return(_size);
	}
	
	public void set_center(float[] pos)
	{
		_center = pos;
		if(have_browser)
			_set_center.setValue(pos);
	}

	public float[] get_center()
	{
		return(_center);
	}
	public double get_enterTime()
	{
		return(_get_enterTime.getValue());
	}

	public double get_exitTime()
	{
		return(_get_exitTime.getValue());
	}

}